﻿from . import _
from Components.ActionMap import ActionMap
from Components.Button import Button
from Components.config import config, ConfigInteger, ConfigNumber, ConfigSelection, ConfigSequence, ConfigSubsection, ConfigYesNo, getConfigListEntry
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.Pixmap import Pixmap, MovingPixmap
from enigma import eTimer, getDesktop
from keymapparser import readKeymap
from Plugins.Plugin import PluginDescriptor
from Screens.InfoBar import InfoBar, MoviePlayer
from Screens.Screen import Screen
from Tools.Directories import fileExists, resolveFilename, SCOPE_PLUGINS
config.plugins.TimeSeekBar = ConfigSubsection()
config.plugins.TimeSeekBar.sensibility = ConfigInteger(default=10, limits=(1, 30))
config.plugins.TimeSeekBar.defaulttime = ConfigInteger(default=5, limits=(1, 60))

if (getDesktop(0).size().width()) > 1280:
	HD = True
else:
	HD = False

class Setup_config(ConfigListScreen, Screen):
	if HD:
		skin = """
<screen title="%s" position="center,960" size="690,120" >
	<widget name="config" position="15,0" size="668,76" itemHeight="38" font="Regular;30" selectionPixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/sel.png" />
	<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/red.png" position="173,114" size="165,3" alphatest="on" />
	<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/green.png" position="353,114" size="165,3" alphatest="on" />
	<widget source="key_red" render="Label" position="173,76" size="165,33" font="Regular;30" halign="center" />
	<widget source="key_green" render="Label" position="353,76" size="165,33" font="Regular;30" halign="center" />
</screen>""" % _("TimeSeekBar Setup")
	else:
		skin = """
<screen title="%s" position="center,640" size="460,80" >
	<widget name="config" position="10,0" size="445,50" selectionPixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/sel.png" />
	<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/red.png" position="115,75" size="110,2" alphatest="on" />
	<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/green.png" position="235,75" size="110,2" alphatest="on" />
	<widget source="key_red" render="Label" position="115,50" size="110,22" font="Regular;20" halign="center" />
	<widget source="key_green" render="Label" position="235,50" size="110,22" font="Regular;20" halign="center" />
</screen>""" % _("TimeSeekBar Setup")

	def __init__(self, session, args = 0):
		self.session = session
		Screen.__init__(self, session)
		self.list = []
		self.list.append(getConfigListEntry(_('Default skip time'), config.plugins.TimeSeekBar.defaulttime))
		self.list.append(getConfigListEntry(_('Sensibility of the search (1-30)'), config.plugins.TimeSeekBar.sensibility))
		ConfigListScreen.__init__(self, self.list)
		self['key_red'] = Button(_('Cancel'))
		self['key_green'] = Button(_('Save'))
		self['setupActions'] = ActionMap(['SetupActions', 'ColorActions'], {'red': self.exit,
		 'green': self.save,
		 'save': self.save,
		 'cancel': self.exit,
		 'ok': self.save}, -2)

	def save(self):
		for x in self['config'].list:
			x[1].save()
		self.close(True, self.session)

	def exit(self):
		for x in self['config'].list:
			x[1].cancel()
		self.close()

class TimeSeekBar(ConfigListScreen, Screen):
	if HD:
		skin = """
<screen title="%s" position="center,960" size="690,120" >
	<widget name="config" position="15,0" size="668,114" itemHeight="38" font="Regular;30" selectionPixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/sel.png" />
	<widget name="cursor" pixmap="skin_default/position_arrow.png" position="145,72" size="12,27" zPosition="3" alphatest="on" />
	<eLabel position="145,96" size="400,6" zPosition="1" backgroundColor="#00555555" />
	<widget source="session.CurrentService" render="PositionGauge" position="145,90" size="400,18" zPosition="2" pointer="PLi-FullHD/dvr/position_pointer1024.png:1536,4" transparent="1" >
		<convert type="ServicePosition">Gauge</convert>
	</widget>
	<widget name="time" position="0,84" size="143,30" font="Regular;30" halign="center" zPosition="2" />
	<widget source="session.CurrentService" render="Label" position="548,84" size="143,30" font="Regular;30" halign="center" zPosition="2" >
		<convert type="ServicePosition">Length,ShowHours</convert>
	</widget>
</screen>""" % _("TimeSeekBar")
	else:
		skin = """
<screen title="%s" position="center,640" size="460,80" >
	<widget name="config" position="10,0" size="445,75" selectionPixmap="/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/images/sel.png" />
	<widget name="cursor" pixmap="skin_default/position_arrow.png" position="97,48" size="8,18" zPosition="3" alphatest="on" />
	<eLabel position="97,65" size="266,4" zPosition="1" backgroundColor="#00555555" />
	<widget source="session.CurrentService" render="PositionGauge" position="97,60" size="266,12" zPosition="2" pointer="PLi-HD/dvr/position_pointer1024.png:1012,0" transparent="1" >
		<convert type="ServicePosition">Gauge</convert>
	</widget>
	<widget name="time" position="0,56" size="95,20" font="Regular;20" halign="center" zPosition="2" />
	<widget source="session.CurrentService" render="Label" position="365,56" size="95,20" font="Regular;20" halign="center" zPosition="2" >
		<convert type="ServicePosition">Length,ShowHours</convert>
	</widget>
</screen>""" % _("TimeSeekBar")

	def __init__(self, session, instance, fwd):
		Screen.__init__(self, session)
		self.session = session
		self.infobarInstance = instance
		self.fwd = fwd
		if isinstance(session.current_dialog, MoviePlayer):
			self.movie = True
		else:
			self.movie = False
		try:
			if isinstance(session.current_dialog, MoviePlayer2):
				self.media = True
			else:
				self.media = False
		except:
			self.media = False
		try:
			if isinstance(session.current_dialog, DVDPlayer):
				self.dvd_pli = True
			else:
				self.dvd_pli = False
		except:
			self.dvd_pli = False
		try:
			if isinstance(session.current_dialog, MyTubePlayer):
				self.mytube = True
			else:
				self.mytube = False
		except:
			self.mytube = False
		try:
			if isinstance(session.current_dialog, YouTubePlayer):
				self.youtube = True
			else:
				self.youtube = False
		except:
			self.youtube = False
		try:
			if isinstance(session.current_dialog, VODplayer):
				self.vod = True
			else:
				self.vod = False
		except:
			self.vod = False
		try:
			if isinstance(session.current_dialog, nVODplayer):
				self.nvod = True
			else:
				self.nvod = False
		except:
			self.nvod = False
		try:
			if isinstance(session.current_dialog, Player):
				self.seasondream = True
			else:
				self.seasondream = False
		except:
			self.seasondream = False
		self.percent = 0.0
		self.length = None
		defaulttime = config.plugins.TimeSeekBar.defaulttime.value
		service = session.nav.getCurrentService()
		if self.fwd == False:
			deftime = - config.plugins.TimeSeekBar.defaulttime.value
		else:
			deftime = config.plugins.TimeSeekBar.defaulttime.value
		xdeftime = int(float(deftime) * 60 * 90000)
		self.initime = xdeftime
		if service:
			self.seek = service.seek()
			if self.seek:
				self.length = self.seek.getLength()
				position = self.seek.getPlayPosition()
				if self.length and position:
					if int(position[1]) > 0:
						if float(self.length[1]) <> 0:
							self.percent = float(position[1]) * 100.0 / float(self.length[1])
						else:
							self.close()	
		self.minuteInput = ConfigNumber(default=defaulttime)
		self.positionEntry = ConfigSelection(choices=[' '], default=' ')
		self.manualInput = ConfigSequence(seperator=':', default=[int(position[1] / 60 / 60 / 90000), int(position[1] / 60 / 90000 % 60), int(position[1] / 90000 % 60)], limits=[(0, 9), (0, 59), (0, 59)])
		if self.fwd:
			txt = _("Jump X minutes forward")
		else:
			txt = _("Jump X minutes back")
		ConfigListScreen.__init__(self, [
			getConfigListEntry(txt, self.minuteInput),
			getConfigListEntry(_('Go to position'), self.manualInput),
			getConfigListEntry(_(' '), self.positionEntry)])
		self['cursor'] = MovingPixmap()
		self['time'] = Label()
		self['actions'] = ActionMap(['WizardActions',
		 'ColorActions',
		 'MenuActions',
		 'DirectionActions'], {'back': self.exit,
		 'menu': self.keyMenu,
		 'red': self.gotostart,
		 'green': self.goto25percent,
		 'yellow': self.goto50percent,
		 'blue': self.goto75percent}, -1)
		self.cursorTimer = eTimer()
		self.cursorTimer.callback.append(self.updateCursor)
		self.onLayoutFinish.append(self.firstStart)
		return

	def firstStart(self):
		try:
			longitud = float(self.length[1])
			if longitud <= 0:
				longitud = 1
			self.percent = float(self.seek.getPlayPosition()[1] + self.initime) * 100.0 / longitud
		except:
			pass

		self.updateCursor()
		self.cursorTimer.start(200, False)
		self['config'].setCurrentIndex(2)

	def updateCursor(self):
		if self.length:
			if self.percent > 100.0:
				self.percent = 100.0
			elif self.percent < 0.0:
				self.percent = 0.0
			if HD:	
				x = 145 + int(4.00 * self.percent)
				posy = self['cursor'].instance.position().y()
				self['cursor'].moveTo(x - 6, posy, 1)
				self['cursor'].startMoving()
				pts = int(float(self.length[1]) / 100.0 * self.percent)
				self['time'].setText('%d:%02d:%02d' % (pts / 60 / 60 / 90000, pts / 60 / 90000 % 60, pts / 90000 % 60))
			else:	
				x = 97 + int(2.66 * self.percent)
				posy = self['cursor'].instance.position().y()
				self['cursor'].moveTo(x - 4, posy, 1)
				self['cursor'].startMoving()
				pts = int(float(self.length[1]) / 100.0 * self.percent)
				self['time'].setText('%d:%02d:%02d' % (pts / 60 / 60 / 90000, pts / 60 / 90000 % 60, pts / 90000 % 60))

	def gotostart(self):
		self.percent = 0.0
		self['config'].setCurrentIndex(2)

	def goto25percent(self):
		self.percent = 25.0
		self['config'].setCurrentIndex(2)

	def goto50percent(self):
		self.percent = 50.0
		self['config'].setCurrentIndex(2)

	def goto75percent(self):
		self.percent = 75.0
		self['config'].setCurrentIndex(2)

	def keyMenu(self):
		self.session.open(Setup_config)

	def exit(self):
		self.cursorTimer.stop()
		ConfigListScreen.saveAll(self)
		self.close()

	def keyOK(self):
		sel = self["config"].getCurrent()[1]
		if sel == self.positionEntry:
			if self.length:
				if self.media:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = int(float(self.length[1]) / 100.0 * self.percent)
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
					MoviePlayer2.doSeekRelative(self.infobarInstance, pts)
				elif self.dvd_pli:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = int(float(self.length[1]) / 100.0 * self.percent)
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
					DVDPlayer.doSeekRelative(self.infobarInstance, pts)
				elif self.mytube:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = int(float(self.length[1]) / 100.0 * self.percent)
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
					MyTubePlayer.doSeekRelative(self.infobarInstance, pts)
				elif self.youtube:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = int(float(self.length[1]) / 100.0 * self.percent)
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
					YouTubePlayer.doSeekRelative(self.infobarInstance, pts)
				elif self.vod:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = int(float(self.length[1]) / 100.0 * self.percent)
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
					VODplayer.doSeekRelative(self.infobarInstance, pts)
				elif self.nvod:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = int(float(self.length[1]) / 100.0 * self.percent)
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
					nVODplayer.doSeekRelative(self.infobarInstance, pts)
				elif self.seasondream:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = int(float(self.length[1]) / 100.0 * self.percent)
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
					Player.doSeekRelative(self.infobarInstance, pts)
				elif self.movie:
					self.seek.seekTo(int(float(self.length[1]) / 100.0 * self.percent))
				else:
					pass
				self.exit()
		elif sel == self.minuteInput:
			pts = self.minuteInput.value * 60 * 90000
			if self.fwd == False:
				pts = -1*pts
			if self.media:
				MoviePlayer2.doSeekRelative(self.infobarInstance, pts)
			elif self.dvd_pli:
				DVDPlayer.doSeekRelative(self.infobarInstance, pts)
			elif self.mytube:
				MyTubePlayer.doSeekRelative(self.infobarInstance, pts)
			elif self.youtube:
				YouTubePlayer.doSeekRelative(self.infobarInstance, pts)
			elif self.vod:
				VODplayer.doSeekRelative(self.infobarInstance, pts)
			elif self.nvod:
				nVODplayer.doSeekRelative(self.infobarInstance, pts)
			elif self.seasondream:
				Player.doSeekRelative(self.infobarInstance, pts)
			elif self.movie:
				MoviePlayer.doSeekRelative(self.infobarInstance, pts)
			else:
				pass
			self.exit()
		elif sel == self.manualInput:
			if self.length:
				if self.dvd_pli:
					oldPosition = self.seek.getPlayPosition()[1]
					newPosition = self.manualInput.value[0] * 60 * 60 * 90000 + self.manualInput.value[1] * 60 * 90000 + self.manualInput.value[2] * 90000
					if newPosition > oldPosition:
						pts = newPosition - oldPosition
					else:
						pts = -1 * (oldPosition - newPosition)
				else:
					self.seek.seekTo(self.manualInput.value[0] * 60 * 60 * 90000 + self.manualInput.value[1] * 60 * 90000 + self.manualInput.value[2] * 90000)
				self.exit()

	def keyLeft(self):
		sel = self['config'].getCurrent()[1]
		if sel == self.positionEntry:
			self.percent -= float(config.plugins.TimeSeekBar.sensibility.value) / 10.0
			if self.percent < 0.0:
				self.percent = 0.0
		else:
			ConfigListScreen.keyLeft(self)
			if sel == self.minuteInput:
				pts = int(float(self.minuteInput.value) * 60 * 90000)
				longitud = float(self.length[1])
				if longitud <= 0:
					longitud = 1
				self.percent = float(self.seek.getPlayPosition()[1] + pts) * 100.0 / longitud
				if self.percent < 0.0:
					self.percent = 0.0
				if self.percent > 100.0:
					self.percent = 100.0

	def keyRight(self):
		sel = self['config'].getCurrent()[1]
		if sel == self.positionEntry:
			self.percent += float(config.plugins.TimeSeekBar.sensibility.value) / 10.0
			if self.percent > 100.0:
				self.percent = 100.0
		else:
			ConfigListScreen.keyRight(self)
			if sel == self.minuteInput:
				pts = int(float(self.minuteInput.value) * 60 * 90000)
				longitud = float(self.length[1])
				if longitud <= 0:
					longitud = 1
				self.percent = float(self.seek.getPlayPosition()[1] + pts) * 100.0 / longitud
				if self.percent < 0.0:
					self.percent = 0.0
				if self.percent > 100.0:
					self.percent = 100.0

	def keyNumberGlobal(self, number):
		sel = self['config'].getCurrent()[1]
		if sel == self.positionEntry:
			self.percent = float(number) * 10.0
		else:
			ConfigListScreen.keyNumberGlobal(self, number)

def timeseekbar(instance, fwd = True):
	if instance and instance.session:
		instance.session.open(TimeSeekBar, instance, fwd)

def timeseekbarBack(instance):
	timeseekbar(instance, False)

keymap = "/usr/lib/enigma2/python/Plugins/Extensions/TimeSeekBar/keymap.xml"
global globalActionMap
readKeymap(keymap)

MoviePlayer.right = timeseekbar
MoviePlayer.left = timeseekbarBack
InfoBar.seekFwdManual = timeseekbar
InfoBar.seekBackManual = timeseekbarBack

if fileExists("/usr/lib/enigma2/python/Screens/DVD.pyo"):
	try:
		from Screens.DVD import DVDPlayer
		DVDPlayer.seekFwdManual = timeseekbar
		DVDPlayer.seekBackManual = timeseekbarBack
	except:
		pass

mediaPlayer = '%s%s' % (resolveFilename(SCOPE_PLUGINS), 'Extensions/MediaPlayer/plugin.pyo')
if fileExists(mediaPlayer):
	try:
		from Plugins.Extensions.MediaPlayer.plugin import MediaPlayer as MoviePlayer2
		MoviePlayer2.seekFwdManual = timeseekbar
		MoviePlayer2.seekBackManual = timeseekbarBack
	except:
		pass

MyTubePlayer = '%s%s' % (resolveFilename(SCOPE_PLUGINS), 'Extensions/MyTube/plugin.pyo')
if fileExists(MyTubePlayer):
	try:
		from Plugins.Extensions.MyTube.plugin import MyTubePlayer
		MyTubePlayer.seekFwdManual = timeseekbar
		MyTubePlayer.seekBackManual = timeseekbarBack
	except:
		pass

YouTubePlayer = '%s%s' % (resolveFilename(SCOPE_PLUGINS), 'Extensions/Y0uTube/plugin.pyo')
if fileExists(YouTubePlayer):
	try:
		from Plugins.Extensions.YouTube.plugin import YouTubePlayer
		YouTubePlayer.seekFwdManual = timeseekbar
		YouTubePlayer.seekBackManual = timeseekbarBack
	except:
		pass

VODPlayer = '%s%s' % (resolveFilename(SCOPE_PLUGINS), 'Extensions/VOD/plugin.pyo')
if fileExists(VODPlayer):
	try:
		from Plugins.Extensions.VOD.plugin import VODplayer
		VODplayer.seekFwdManual = timeseekbar
		VODplayer.seekBackManual = timeseekbarBack
	except:
		pass

nStreamVODPlayer = '%s%s' % (resolveFilename(SCOPE_PLUGINS), 'Extensions/nStreamVOD/plugin.pyo')
if fileExists(nStreamVODPlayer):
	try:
		from Plugins.Extensions.nStreamVOD.plugin import nVODplayer
		nVODplayer.seekFwdManual = timeseekbar
		nVODplayer.seekBackManual = timeseekbarBack
	except:
		pass

cScVODPlayer = '%s%s' % (resolveFilename(SCOPE_PLUGINS), 'Extensions/cScVOD/plugin.pyo')
if fileExists(cScVODPlayer):
	try:
		from Plugins.Extensions.cScVOD.plugin import VODplayer
		VODplayer.seekFwdManual = timeseekbar
		VODplayer.seekBackManual = timeseekbarBack
	except:
		pass

SeasondreamPlayer = '%s%s' % (resolveFilename(SCOPE_PLUGINS), 'Extensions/Seasondream/plugin.pyo')
if fileExists(SeasondreamPlayer):
	try:
		from Plugins.Extensions.Seasondream.Player import Player
		Player.seekFwdManual = timeseekbar
		Player.seekBackManual = timeseekbarBack
	except:
		pass

def Plugins(**kwargs):
	return []
